@@ -93,7 +93,13 @@ rabbit_opts = [
                 'an exception will be raised.'),
     cfg.BoolOpt('heartbeat_in_pthread',
                 default=False,
-                help="Run the health check heartbeat thread "
+                deprecated_for_removal=True,
+                deprecated_reason='The option is related to Eventlet which '
+                                  'will be removed. In addition this has '
+                                  'never worked as expected with services '
+                                  'using eventlet for core service framework.',
+                help="(DEPRECATED) It is recommend not to use this option "
+                     "anymore. Run the health check heartbeat thread "
                      "through a native python thread by default. If this "
                      "option is equal to False then the health check "
                      "heartbeat will inherit the execution model "
@@ -246,8 +252,10 @@ rabbit_opts = [
                 default=False,
                 help='Should we use consistant queue names or random ones'),
     cfg.StrOpt('hostname',
+               sample_default='node1.example.com',
                default=socket.gethostname(),
-               help='Hostname used by queue manager'),
+               help='Hostname used by queue manager. Defaults to the value '
+               'returned by socket.gethostname().'),
     cfg.StrOpt('processname',
                default=os.path.basename(sys.argv[0]),
                help='Process name used by queue manager'),
@@ -397,6 +405,8 @@ class Consumer(object):
             durable=self.durable,
             auto_delete=self.exchange_auto_delete)
         self.enable_cancel_on_failover = enable_cancel_on_failover
+        self.rabbit_stream_fanout = rabbit_stream_fanout
+        self.next_stream_offset = "last"
 
     def _declare_fallback(self, err, conn, consumer_arguments):
         """Fallback by declaring a non durable queue.
@@ -431,6 +441,13 @@ class Consumer(object):
             )
             self.queue.declare()
 
+    def reset_stream_offset(self):
+        if not self.rabbit_stream_fanout:
+            return
+        LOG.warn("Reset consumer for queue %s next offset was at %s.",
+                 self.queue_name, self.next_stream_offset)
+        self.next_stream_offset = "last"
+
     def declare(self, conn):
         """Re-declare the queue after a rabbit (re)connect."""
 
@@ -439,6 +456,10 @@ class Consumer(object):
             consumer_arguments = {
                 "x-cancel-on-ha-failover": True}
 
+        if self.rabbit_stream_fanout:
+            consumer_arguments = {
+                "x-stream-offset": self.next_stream_offset}
+
         self.queue = kombu.entity.Queue(
             name=self.queue_name,
             channel=conn.channel,
@@ -451,8 +472,13 @@ class Consumer(object):
         )
 
         try:
-            LOG.debug('[%s] Queue.declare: %s',
-                      conn.connection_id, self.queue_name)
+            if self.rabbit_stream_fanout:
+                LOG.info('[%s] Stream Queue.declare: %s after offset %s',
+                         conn.connection_id, self.queue_name,
+                         self.next_stream_offset)
+            else:
+                LOG.debug('[%s] Queue.declare: %s',
+                          conn.connection_id, self.queue_name)
             try:
                 self.queue.declare()
             except amqp_ex.PreconditionFailed as err:
@@ -462,7 +488,9 @@ class Consumer(object):
                 # from each others. RabbitMQ will reject the services
                 # that try to create it with a configuration that differ
                 # from the one used first.
-                LOG.warning(err)
+                LOG.warning('[%s] Queue %s could not be declared probably '
+                            'because of conflicting configurations: %s',
+                            conn.connection_id, self.queue_name, err)
                 self._declare_fallback(err, conn, consumer_arguments)
             except amqp_ex.NotFound as ex:
                 # NOTE(viktor.krivak): This exception is raised when
@@ -594,6 +622,12 @@ class Consumer(object):
 
         Messages that are processed and ack'ed.
         """
+        if self.rabbit_stream_fanout:
+            offset = message.headers.get("x-stream-offset")
+            if offset is not None:
+                LOG.debug("Stream for %s current offset: %s",
+                          self.queue_name, offset)
+                self.next_stream_offset = offset + 1
 
         m2p = getattr(self.queue.channel, 'message_to_python', None)
         if m2p:
@@ -1062,11 +1096,24 @@ class Connection(object):
             info = {'err_str': exc, 'sleep_time': interval}
             info.update(self._get_connection_info(conn_error=True))
 
+            if self.rabbit_stream_fanout and 'Basic.cancel' in str(exc):
+                # This branch allows for consumer offset reset
+                # in the unlikely case consumers are cancelled. This may
+                # happen, for example, when we delete the stream queue.
+                # We need to start consuming from "last" because the stream
+                # offset maybe reset.
+                LOG.warn('[%s] Basic.cancel received. '
+                         'Resetting consumers offsets to last.',
+                         self.connection_id)
+                for consumer in self._consumers:
+                    consumer.reset_stream_offset()
+
             if 'Socket closed' in str(exc):
                 LOG.error('[%(connection_id)s] AMQP server'
                           ' %(hostname)s:%(port)s closed'
                           ' the connection. Check login credentials:'
                           ' %(err_str)s', info)
+
             else:
                 LOG.error('[%(connection_id)s] AMQP server on '
                           '%(hostname)s:%(port)s is unreachable: '
