@@ -122,6 +122,7 @@ A simple example of an RPC server with multiple endpoints might be::
 
 import logging
 import sys
+import time
 
 from oslo_messaging import exceptions
 from oslo_messaging.rpc import dispatcher as rpc_dispatcher
@@ -151,6 +152,12 @@ class RPCServer(msg_server.MessageHandlingServer):
 
     def _process_incoming(self, incoming):
         message = incoming[0]
+        rpc_method = message.message.get('method')
+        start = time.time()
+        LOG.debug("Receive incoming message with id %(msg_id)s and "
+                  "method: %(method)s.",
+                  {"msg_id": message.msg_id,
+                   "method": rpc_method})
 
         # TODO(sileht): We should remove that at some point and do
         # this directly in the driver
@@ -168,7 +175,7 @@ class RPCServer(msg_server.MessageHandlingServer):
             # by another exception raised by a log handler during
             # LOG.debug(). So keep a copy and delete it later.
             failure = e.exc_info
-            LOG.debug(u'Expected exception during message handling (%s)', e)
+            LOG.debug('Expected exception during message handling (%s)', e)
         except rpc_dispatcher.NoSuchMethod as e:
             failure = sys.exc_info()
             if e.method.endswith('_ignore_errors'):
@@ -182,8 +189,19 @@ class RPCServer(msg_server.MessageHandlingServer):
         try:
             if failure is None:
                 message.reply(res)
+                LOG.debug("Replied success message with id %(msg_id)s and "
+                          "method: %(method)s. Time elapsed: %(elapsed).3f",
+                          {"msg_id": message.msg_id,
+                           "method": rpc_method,
+                           "elapsed": (time.time() - start)})
             else:
                 message.reply(failure=failure)
+                LOG.debug("Replied failure for incoming message with "
+                          "id %(msg_id)s and method: %(method)s. "
+                          "Time elapsed: %(elapsed).3f",
+                          {"msg_id": message.msg_id,
+                           "method": rpc_method,
+                           "elapsed": (time.time() - start)})
         except exceptions.MessageUndeliverable as e:
             LOG.exception(
                 "MessageUndeliverable error, "
@@ -200,7 +218,8 @@ class RPCServer(msg_server.MessageHandlingServer):
 
 
 def get_rpc_server(transport, target, endpoints,
-                   executor=None, serializer=None, access_policy=None):
+                   executor=None, serializer=None, access_policy=None,
+                   server_cls=RPCServer):
     """Construct an RPC server.
 
     :param transport: the messaging transport
@@ -217,10 +236,12 @@ def get_rpc_server(transport, target, endpoints,
     :param access_policy: an optional access policy.
            Defaults to DefaultRPCAccessPolicy
     :type access_policy: RPCAccessPolicyBase
+    :param server_cls: The server class to instantiate
+    :type server_cls: class
     """
     dispatcher = rpc_dispatcher.RPCDispatcher(endpoints, serializer,
                                               access_policy)
-    return RPCServer(transport, target, dispatcher, executor)
+    return server_cls(transport, target, dispatcher, executor)
 
 
 def expected_exceptions(*exceptions):
