@@ -1,4 +1,3 @@
-
 # Copyright 2013 Red Hat, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -19,6 +18,7 @@ import sys
 import uuid
 
 import fixtures
+from kombu import connection
 from oslo_serialization import jsonutils
 from oslo_utils import strutils
 from oslo_utils import timeutils
@@ -36,6 +36,7 @@ from oslo_messaging import serializer as msg_serializer
 from oslo_messaging.tests import utils as test_utils
 from unittest import mock
 
+
 load_tests = testscenarios.load_tests_apply_scenarios
 
 
@@ -122,7 +123,7 @@ class TestMessagingNotifier(test_utils.BaseTestCase):
     ]
 
     _context = [
-        ('ctxt', dict(ctxt={'user': 'bob'})),
+        ('ctxt', dict(ctxt=test_utils.TestContext(user_name='bob'))),
     ]
 
     _retry = [
@@ -229,6 +230,80 @@ class TestMessagingNotifier(test_utils.BaseTestCase):
 TestMessagingNotifier.generate_scenarios()
 
 
+class TestMessagingNotifierRetry(test_utils.BaseTestCase):
+
+    class TestingException(BaseException):
+        pass
+
+    def test_notifier_retry_connection_fails_rabbit(self):
+        """This test sets a small retry number for notification sending and
+        configures a non reachable message bus. The expectation that after the
+        configured number of retries the driver gives up the message sending.
+        """
+        self.config(
+            driver=["messagingv2"],
+            topics=["test-retry"],
+            retry=2,
+            group="oslo_messaging_notifications")
+        self.config(
+            # just to speed up the test execution
+            rabbit_retry_backoff=0,
+            group="oslo_messaging_rabbit")
+        transport = oslo_messaging.get_notification_transport(
+            self.conf, url='rabbit://')
+        notifier = oslo_messaging.Notifier(transport)
+
+        orig_establish_connection = connection.Connection._establish_connection
+        calls = []
+
+        def wrapped_establish_connection(*args, **kwargs):
+            if len(calls) > 2:
+                raise self.TestingException(
+                    "Connection should only be retried twice due to "
+                    "configuration")
+            else:
+                calls.append((args, kwargs))
+                orig_establish_connection(*args, **kwargs)
+
+        with mock.patch(
+            'kombu.connection.Connection._establish_connection',
+            new=wrapped_establish_connection
+        ):
+            with mock.patch(
+                'oslo_messaging.notify.messaging.LOG.exception'
+            ) as mock_log:
+                notifier.info(test_utils.TestContext(), "test", {})
+
+        # one normal call plus two retries
+        self.assertEqual(3, len(calls))
+        # the error was caught and logged
+        mock_log.assert_called_once()
+
+    def test_notifier_retry_connection_fails_kafka(self):
+        """This test sets a small retry number for notification sending and
+        configures a non reachable message bus. The expectation that after the
+        configured number of retries the driver gives up the message sending.
+        """
+
+        self.config(
+            driver=["messagingv2"],
+            topics=["test-retry"],
+            retry=2,
+            group='oslo_messaging_notifications')
+
+        transport = oslo_messaging.get_notification_transport(
+            self.conf, url='kafka://')
+
+        notifier = oslo_messaging.Notifier(transport)
+
+        # Kafka's message producer interface is async, and there is no way
+        # from the oslo interface to force sending a pending message. So this
+        # call simply returns without i) failing to deliver the message to
+        # the non existent kafka bus ii) retrying the message delivery twice
+        # as the configuration requested it.
+        notifier.info(test_utils.TestContext(), "test", {})
+
+
 class TestSerializer(test_utils.BaseTestCase):
 
     def setUp(self):
@@ -254,12 +329,13 @@ class TestSerializer(test_utils.BaseTestCase):
         mock_utcnow.return_value = datetime.datetime.utcnow()
 
         serializer.serialize_context = mock.Mock()
-        serializer.serialize_context.return_value = dict(user='alice')
+        serializer.serialize_context.return_value = dict(user_name='alice')
 
         serializer.serialize_entity = mock.Mock()
         serializer.serialize_entity.return_value = 'sbar'
 
-        notifier.info(dict(user='bob'), 'test.notify', 'bar')
+        ctxt = test_utils.TestContext(user_name='bob')
+        notifier.info(ctxt, 'test.notify', 'bar')
 
         message = {
             'message_id': str(message_id),
@@ -270,13 +346,13 @@ class TestSerializer(test_utils.BaseTestCase):
             'timestamp': str(timeutils.utcnow()),
         }
 
-        self.assertEqual([(dict(user='alice'), message, 'INFO', -1)],
+        self.assertEqual([(dict(user_name='alice'), message, 'INFO', -1)],
                          _impl_test.NOTIFICATIONS)
 
-        uuid.uuid4.assert_called_once_with()
-        serializer.serialize_context.assert_called_once_with(dict(user='bob'))
-        serializer.serialize_entity.assert_called_once_with(dict(user='bob'),
-                                                            'bar')
+        # NOTE(JayF): This is also called when we create a TestContext
+        uuid.uuid4.assert_has_calls([mock.call(), mock.call()])
+        serializer.serialize_context.assert_called_once_with(ctxt)
+        serializer.serialize_entity.assert_called_once_with(ctxt, 'bar')
 
 
 class TestNotifierTopics(test_utils.BaseTestCase):
@@ -335,9 +411,10 @@ class TestLogNotifier(test_utils.BaseTestCase):
         with mock.patch.object(logging, 'getLogger') as gl:
             gl.return_value = logger
 
-            notifier.info({}, 'test.notify', 'bar')
+            notifier.info(test_utils.TestContext(), 'test.notify', 'bar')
 
-            uuid.uuid4.assert_called_once_with()
+            # NOTE(JayF): TestContext calls this, too
+            uuid.uuid4.assert_has_calls([mock.call(), mock.call()])
             logging.getLogger.assert_called_once_with(
                 'oslo.messaging.notification.test.notify')
 
@@ -592,7 +669,7 @@ group_1:
         with mock.patch.object(self.router, 'plugin_manager') as pm:
             with mock.patch.object(self.router, '_get_drivers_for_message',
                                    drivers_mock):
-                self.notifier.info({}, 'my_event', {})
+                self.notifier.info(test_utils.TestContext(), 'my_event', {})
                 self.assertEqual(sorted(['rpc', 'foo']),
                                  sorted(pm.map.call_args[0][6]))
 
@@ -630,13 +707,13 @@ group_1:
                             return_value=pm):
                 with mock.patch('oslo_messaging.notify.'
                                 '_impl_routing.LOG'):
-
-                    self.notifier.info({}, 'my_event', {})
+                    cxt = test_utils.TestContext()
+                    self.notifier.info(cxt, 'my_event', {})
                     self.assertFalse(bar_driver.info.called)
                     rpc_driver.notify.assert_called_once_with(
-                        {}, mock.ANY, 'INFO', -1)
+                        cxt, mock.ANY, 'INFO', -1)
                     rpc2_driver.notify.assert_called_once_with(
-                        {}, mock.ANY, 'INFO', -1)
+                        cxt, mock.ANY, 'INFO', -1)
 
 
 class TestNoOpNotifier(test_utils.BaseTestCase):
