@@ -12,11 +12,10 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-
 from oslo_messaging._drivers import common
 from oslo_messaging import _utils as utils
 from oslo_messaging.tests import utils as test_utils
-from six.moves import mock
+from unittest import mock
 
 
 class VersionIsCompatibleTestCase(test_utils.BaseTestCase):
@@ -97,3 +96,17 @@ class TimerTestCase(test_utils.BaseTestCase):
         remaining = t.check_return(callback, 1, a='b')
         self.assertEqual(0, remaining)
         callback.assert_called_once_with(1, a='b')
+
+    @mock.patch('oslo_utils.timeutils.now')
+    def test_reset(self, now):
+        now.return_value = 0
+        t = common.DecayingTimer(3)
+        t.start()
+
+        now.return_value = 1
+        remaining = t.check_return()
+        self.assertEqual(2, remaining)
+
+        t.restart()
+        remaining = t.check_return()
+        self.assertEqual(3, remaining)
